#!/bin/bash

#
# ScoDoc:  restore data (saved by save_scodoc9_data) into current install
# 
#  Utile pour migrer ScoDoc 9 d'un serveur a un autre
#  A executer en tant que root sur le nouveau serveur
#
# E. Viennet, Sept 2021
#
SCRIPT_DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" &> /dev/null && pwd )"
source "$SCRIPT_DIR/config.sh"
source "$SCRIPT_DIR/utils.sh"

# Ce script doit tourner comme "root"
check_uid_root "$0"

# Usage
usage() {
  echo "Usage: $0 [ --keep-env ] archive"
  echo "Exemple: $0 /tmp/mon-scodoc.tgz"
  echo "OPTION"
  echo "--keep_env garde la configuration courante"
  exit 1
}

# détection des options -> ($SRC, $KEEP_ENV) sinon usage
# 1 ou 2 parametres et si 2, l un des deux doit être -keep-env
if [ $# -eq 1 ]; then
  KEEP=""
  SRC="$1"
elif [ $# -eq 2 ]; then
  if [ "$1" = "--keep-env" ]; then
    KEEP=1
    SRC=$2
  elif [ "$2" = "--keep-env" ]; then
    KEEP=1
    SRC="$1"
  else
    usage
  fi
else
  usage
fi

if [ ! -e $SRC ]; then
  echo "le fichier de restauration ($SRC) n'existe pas: Abandon de la restauration"
  exit 1
fi

if [ -z $KEEP ]; then
  echo "restauration des données tout en conservant la configuration actuelle"
  DB_CURRENT=$(su -c "(cd $SCODOC_DIR && source venv/bin/activate && flask scodoc-database -n)" "$SCODOC_USER")
  DB_DEST="$DB_CURRENT"
else
  echo "restauration des données et de la configuration originale (de production)"
  DB_DEST="SCODOC"
fi
DB_DUMP="${SCODOC_VAR_DIR}"/SCODOC.dump

# Safety check
echo "Ce script va remplacer les donnees de votre installation ScoDoc par celles"
echo "enregistrées dans le fichier fourni."
echo "Ce fichier doit avoir ete cree par le script save_scodoc9_data.sh."
echo 
echo "Attention: TOUTES LES DONNEES DE CE SCODOC SERONT REMPLACEES !"
echo "Notamment, tous les utilisateurs et departements existants seront effaces !"
echo
echo "La base SQL $DB_CURRENT sera effacée et remplacée !!!"
if [ ! -z $KEEP ]; then
   echo -n "Assurez-vous d\'avoir arrété le serveur scodoc."
fi
SCODOC_VAR_DIR_OLD="${SCODOC_VAR_DIR}".old
if [ -e "$SCODOC_VAR_DIR" ] && [ -e "$SCODOC_VAR_DIR_OLD" ]; then
  old_date=$(stat --printf %z $SCODOC_VAR_DIR_OLD)
  echo "un ancien répertoire de sauvegarde $SCODOC_VAR_DIR_OLD du $old_date va être écrasé"
fi

echo
echo -n "Voulez vous poursuivre cette opération ? (y/n) [n]"
read -r ans
if [ ! "$(norm_ans "$ans")" = 'Y' ]; then
  echo "Annulation par l\'utilisateur"
  exit 1
fi

# -- Stop ScoDoc
if [ $KEEP -ne 1 ]; then
   echo "Arrêt de scodoc9..."
   systemctl stop scodoc9
fi

# Clear caches
echo "Purge des caches..."
su -c "(cd $SCODOC_DIR && source venv/bin/activate && flask clear-cache)" "$SCODOC_USER" || die "Erreur purge cache scodoc9"

# Déplace scodoc-data s'il existe
if [ -e "$SCODOC_VAR_DIR" ]; then
  if [ -e "$SCODOC_VAR_DIR_OLD" ]; then
    echo "$SCODOC_VAR_DIR_OLD existe: le supprime"
    rm -rf $SCODCO_VAR_DIR_OLD || die "Erreur effacement d\'une ancienne sauvegarde"
  fi
  echo "$SCODOC_VAR_DIR existe: le renomme en $SCODOC_VAR_DIR_OLD"
  mv "$SCODOC_VAR_DIR" "$SCODOC_VAR_DIR_OLD" || die "Erreur renommage scodoc-data"
fi
mkdir "$SCODOC_VAR_DIR"

# -- Ouverture archive
echo "Ouverture archive $SRC..."
(cd $(dirname "$SCODOC_VAR_DIR"); tar xfz "$SRC") || die "Error opening archive"

# -- Ckeck/fix owner
echo "Vérification du propriétaire..."
chown -R "${SCODOC_USER}:${SCODOC_GROUP}" "${SCODOC_VAR_DIR}" || die "Error chowning ${SCODOC_VAR_DIR}"

# --- Reconstruction de la base cible ($DB_DEST)
nb=$(su -c "psql -l" "$SCODOC_USER" | awk '{print $1}' | grep -c -x "$DB_DEST")
if [ "$nb" -gt 0 ]; then
  echo "Suppression de la base $DB_DEST..."
  su -c "dropdb $DB_DEST" "$SCODOC_USER" || die "Erreur destruction db"
fi
su -c "createdb $DB_DEST" "$SCODOC_USER" || die "Erreur création db"

if [ ! -z $KEEP_ENV ]; then
   echo "conservation de la configuration actuelle"
   mv "$SCODOC_VAR_DIR"/.env "$SCODOC_VAR_DIR"/.env.initial
   cp "$SCODOC_VAR_DIR_OLD"/.env "$SCODOC_VAR_DIR"/.env
   echo "récupération des données..."
   su -c "pg_restore -f - $DB_DUMP | psql -q $DB_DEST" "$SCODOC_USER" >/dev/null || die "Erreur chargement/renommage de la base SQL"
   su -c "(cd $SCODOC_DIR && source venv/bin/activate && flask db upgrade)" "$SCODOC_USER"
   echo "redémarrez scodoc selon votre configuration"
else
# -- Apply migrations if needed (only on "production" database, = SCODOC sauf config particulière)
   echo "restauration environnement de production (initial)"
   echo "Chargement de la base SQL..."
   su -c "pg_restore -d $DB_DEST $DB_DUMP" "$SCODOC_USER" || die "Erreur chargement de la base SQL"
   export FLASK_ENV="production"   # peut-être pas utile? : .env a été recopié
   su -c "(cd $SCODOC_DIR && source venv/bin/activate && flask db upgrade)" "$SCODOC_USER"
# -- Start ScoDoc
   systemctl start scodoc9
fi


echo "Terminé."
